/***********************************************************\
*                                                           *
*                         Osciloskop                        *
*                                                           *
* (c) Radim Pechal, radim.pechal@seznam.cz                  *
* http://radim.xf.cz/                                       *
*                                                           *
* v. 1.0 z data 4. 3. 2007                                  *
*                                                           *
\***********************************************************/


#include <hidef.h>
#include "derivative.h"
volatile byte _NVICSTRM @0x0000FFAF;
#define NVICSTRM _NVICSTRM
#define hi_baud 9600
#define fei1_spd 8500000/16/hi_baud

byte pole[250];           // Pole hodnot namerenych AD0
byte trigger=0x64;        // Trigrovaci uroven AD prevodniku

byte stav=0x06;           // Nastaveni osciloskopu
// stav.0 hrana 0=vzestupna; 1=sestupna
// stav.1 0=unipolar 1=bipolar
// stav.2 0=synchro 1=free
// stav.3 0=mereni 1=hold
// stav.4 0=free run 1=one shot
// stav.5 0=DC 1=AC

byte timebase=7;
// timebase == 0 ..  50us/div
// timebase == 1 .. 100us/div
// timebase == 2 .. 200us/div
// timebase == 3 .. 500us/div
// timebase == 4 ..   1ms/div
// timebase == 5 ..   2ms/div
// timebase == 6 ..   5ms/div
// timebase == 7 ..  10ms/div
// timebase == 8 ..  20ms/div
// timebase == 9 ..  50ms/div
// timebase ==10 .. 100ms/div
// timebase ==11 .. 200ms/div
// timebase ==12 .. 500ms/div
// timebase ==13 ..   1 s/div
// timebase ==14 ..   2 s/div

byte gain=6;
// gain == 0 ..  10mV/div  zesileni 50   vstup  1 : 1
// gain == 1 ..  20mV/div  zesileni 25   vstup  1 : 1
// gain == 2 ..  50mV/div  zesileni 10   vstup  1 : 1
// gain == 3 .. 100mV/div  zesileni  5   vstup  1 : 1
// gain == 4 .. 200mV/div  zesileni  2.5 vstup  1 : 1
// gain == 5 .. 500mV/div  zesileni  1   vstup  1 : 1
// gain == 6 ..   1 V/div  zesileni 50   vstup  1 : 100
// gain == 7 ..   2 V/div  zesileni 25   vstup  1 : 100
// gain == 8 ..   5 V/div  zesileni 10   vstup  1 : 100
// gain == 9 ..  10 V/div  zesileni  5   vstup  1 : 100
// gain ==10 ..  20 V/div  zesileni  2.5 vstup  1 : 100
// gain ==11 ..  50 V/div  zesileni  1   vstup  1 : 100

byte Xhold;    // Promenna urcujici X pozici pro Hold

byte screen=0; // Pomocna promenna pro Free

byte mergain;
byte mertimebase;

byte obrX=0xF;
byte obrY=0xF;

void feedCOP() {
  if (PTAD_PTAD5 == 1)
    SRS = 1;			  
  if (SPMSC1_LVDF == 1) {
    PTBD = PTBD^0x10; 
    SPMSC1_LVDACK = 1;
  }
}

// Function pro prepinani mezi vnitnimi hodinami
void ics_intclk(byte C1, byte C2) {
byte i;

  if (NVICSTRM != 0xFF) {
    ICSTRM = NVICSTRM;					        // nacteni trimovaciho registru
  }
  ICSC1 = C1;
  for (i=0x10;i>0;i--){
    feedCOP();
  }
  ICSC2 = C2;														
  while (ICSC1_CLKS != ICSSC_CLKST) {}  
}



void SendChar(char znak) {
  byte i;

  i = SCIS1;                    // Nastartovani serioveho prenosu
  SCID = znak;                  // Odeslani znaku
  while(!SCIS1_TDRE){           // Ocekavani pro odeslani znaku
    feedCOP();
  };
  while(!SCIS1_TC){
    feedCOP();
  };
} 


void SendMsg(char msg[]) {
  byte ix=0;	                
  byte dummy;	        
  byte nxt_char;
    dummy = SCIS1;              
  nxt_char = msg[ix++];
  while(nxt_char != 0x00) {
    SCID = nxt_char;            
    nxt_char = msg[ix++];
    while(!SCIS1_TDRE){
      feedCOP();
    };
  }
  while(!SCIS1_TC){
    feedCOP();
  };
}



// ********************************************************
// *                   FUNKCE PRO CEKANI                  *
// ********************************************************


void Cekej10ms(void) 
{
  byte i,j;

  for(i=0; i<19; i++)
    for(j=0; j<255; j++) asm{
        STA     SRS   
    }
}


void Cekej20ms(void) 
{
  byte i,j;

  for(i=0; i<39; i++)
    for(j=0; j<255; j++) asm{
        STA     SRS   
    }
}


void Cekej100ms(void){
  byte i,j;

  for(i=0; i<196; i++)
    for(j=0; j<255; j++) asm{
        STA     SRS   
    }
}


// ********************************************************
// *                   FUNKCE PRO MERENI                  *
// ********************************************************

byte ADquickSynchro() {
  word ukazat;
  byte counter1, counter2, counter3;

  counter1 = 0;
  counter2 = 0;
  counter3 = 10;

  // Pretypuji adresu pole (prekladac v ASM casti to odmital)
  ukazat=(unsigned int)pole;

  if (stav & 0x1) 
  asm{
  ; Zakaz preruseni
          CLI
  ; Priprav ukazatel na zacatek pole
          LDHX   ukazat  
  ; Cekani na komparacni uroven pri vzestupne hrane
          LDA    trigger                   
  WAITL:  STA    SRS
          DBNZ   counter1,WAITLA
          DBNZ   counter2,WAITLA
          DBNZ   counter3,WAITLA
          JMP    ERTRIG
  WAITLA: CMP    ADCRL          
          BLO    WAITL
  ; ... a pak vyskocit nad komparacni uroven          
  WAITH:  STA    SRS
          DBNZ   counter1,WAITHA
          DBNZ   counter2,WAITHA
          DBNZ   counter3,WAITHA
          JMP    ERTRIG
  WAITHA: CMP    ADCRL
          BHS    WAITH                    
  }
  else
  asm{
  ; Zakaz preruseni
          CLI
  ; Priprav ukazatel na zacatek pole
          LDHX   ukazat  
  ; Cekani na komparacni uroven pri sestupne hrane
          LDA    trigger
  WAITSH: STA    SRS
          DBNZ   counter1,WAITSHA
          DBNZ   counter2,WAITSHA
          DBNZ   counter3,WAITSHA
          JMP    ERTRIG
  WAITSHA:CMP    ADCRL          
          BHS    WAITSH
  ; ... a pak spadnout pod komparacni uroven          
  WAITSL: STA    SRS
          DBNZ   counter1,WAITSLA
          DBNZ   counter2,WAITSLA
          DBNZ   counter3,WAITSLA
          JMP    ERTRIG
  WAITSLA:CMP    ADCRL
          BLO    WAITSL
  }
  asm {
  ; "A" registr pouziju jako pocitadlo      
  MERENI: LDA    #$FA       ; Pocitadlo nastaveno na 250
  ; A zaznamenam data (po 17 Bus cyklech) do pameti
  LOOP:   MOV    ADCRL,X+   ; 5 Bus cycles
          NOP               ; 1 (1)
          NOP               ; 1 (2)
          NOP               ; 1 (3)
          NOP               ; 1 (4)
          NOP               ; 1 (5)
          NOP               ; 1 (6)
          NOP               ; 1 (7)
          NOP               ; 1 (8)
          DBNZA   LOOP      ; 4 Bus cycles
  ; Povolim preruseni
          SEI
  ; Ulozeni navratove hodnoty do counter1
          LDA     #0x0
          JMP     KONEC
  ERTRIG: LDA     #0x1
  KONEC:  STA     counter1
  }

  return(counter1);
}



byte ADnormalSynchro(byte speed) {
  word ukazat;
  byte i, counter1, counter2, counter3;

  counter1 = 0;
  counter2 = 0;
  counter3 = 10;

  // Pretypuji adresu pole (prekladac v ASM casti to odmital)
  ukazat=(unsigned int)pole;

  if (stav & 0x1) 
  asm{
  ; Zakaz preruseni
          CLI
  ; Priprav ukazatel na zacatek pole
          LDHX   ukazat  
  ; Cekani na komparacni uroven pri vzestupne hrane
          LDA    trigger                   
  WAITL:  STA    SRS
          DBNZ   counter1,WAITLA
          DBNZ   counter2,WAITLA
          DBNZ   counter3,WAITLA
          JMP    ERTRIG
  WAITLA: CMP    ADCRL          
          BLO    WAITL
  ; ... a pak vyskocit nad komparacni uroven          
  WAITH:  STA    SRS
          DBNZ   counter1,WAITHA
          DBNZ   counter2,WAITHA
          DBNZ   counter3,WAITHA
          JMP    ERTRIG
  WAITHA: CMP    ADCRL
          BHI    WAITH                    
  }
  else
  asm{
  ; Zakaz preruseni
          CLI
  ; Priprav ukazatel na zacatek pole
          LDHX   ukazat  
  ; Cekani na komparacni uroven pri sestupne hrane
          LDA    trigger
  WAITSH: STA    SRS
          DBNZ   counter1,WAITSHA
          DBNZ   counter2,WAITSHA
          DBNZ   counter3,WAITSHA
          JMP    ERTRIG
  WAITSHA:CMP    ADCRL          
          BHI    WAITSH
  ; ... a pak spadnout pod komparacni uroven          
  WAITSL: STA    SRS
          DBNZ   counter1,WAITSLA
          DBNZ   counter2,WAITSLA
          DBNZ   counter3,WAITSLA
          JMP    ERTRIG
  WAITSLA:CMP    ADCRL
          BLO    WAITSL
  }


  asm {
  ; "A" registr pouziju jako pocitadlo      
          LDA    #$FA       ; Pocitadlo nastaveno na 250
  ; A zaznamenam data (po 17 Bus cyklech) do pameti
    LOOPL:  MOV    ADCRL,X+   ; 5 Bus cycles

            STA    i          ; 5
            LDA    speed      ; 5 (15)

            NOP               ; 1 (1)
            NOP               ; 1 (2)
            NOP               ; 1 (3)
            NOP               ; 1 (4)
            NOP               ; 1 (5)
            NOP               ; 1 (6)
            NOP               ; 1 (7)
            NOP               ; 1 (8)
            NOP               ; 1 (9)
            DECA              ; 1
            BEQ    LOOPLC     ; 3 (25)
            



    LOOPS:  NOP               ; 1 (1)
            NOP               ; 1 (2)
            NOP               ; 1 (3)
            NOP               ; 1 (4)
            NOP               ; 1 (5)
            NOP               ; 1 (6)
            NOP               ; 1 (7)
            NOP               ; 1 (8)
            NOP               ; 1 (9)
            NOP               ; 1 (10)
            NOP               ; 1 (11)
            NOP               ; 1 (12)
            NOP               ; 1 (13)
            DBNZA   LOOPS     ; 4


    LOOPLC: LDA     i         ; 5
            DBNZA   LOOPL     ; 4 Bus cycles (34)
  ; Povolim preruseni
          SEI
  ; Ulozeni navratove hodnoty do counter1
          LDA     #0x0
          JMP     KONEC
  ERTRIG: LDA     #0x1
  KONEC:  STA     counter1
  }
  return(counter1);
}

Bool ADhigher(void){
byte i;

  for (i = 0; i<10; i++){
    if (ADCRL<trigger) return(0);
    asm{
              STA   SRS
              STA   SRS
              STA   SRS
              STA   SRS
              STA   SRS
    }
  }  
  return(1);
}

Bool ADlower(void){
byte i;

  for (i = 0; i<10; i++){
    if (ADCRL>trigger) return(0);
    asm{
              STA   SRS
              STA   SRS
              STA   SRS
              STA   SRS
              STA   SRS
    }
  }  
  return(1);
}


byte ADslowSynchro(void){
byte i;
word counter;



  counter = 10000;
  if (stav & 0x01){
    while ((ADhigher())&&(counter>0)) counter--;
    if (counter) while ((ADlower())&&(counter>0)) counter--;
  } else {
    while ((ADlower())&&(counter>0)) counter--;
    if (counter) while ((ADhigher())&&(counter>0)) counter--;
  }
  if (counter){    
// Nastaveni casovace
    MTIMSC = 0x30;
    MTIMMOD = 0x6A;
    MTIMCLK = 0x06;  
    MTIMSC = 0x00;

    for (i = 0; i<250; i++){
      pole[i] = ADCRL;
      while (!(MTIMSC & 0x80)) SRS = 42;
      MTIMSC = 0;
     
    }
    MTIMSC = 0x30;
  }
 return(!((byte)counter));  

}


byte ADveryslowSynchro(byte t){
byte i,n;
word counter;

// Nastaveni zobrazovani nezobrazovane obrazovky
  if (!(stav & 0x08)){
    SendChar(0x1B);  
    SendChar('V');
    if (screen == 1) SendChar('1');
      else SendChar('2');  
  }
  Cekej20ms();

  SendChar(0x1B);
  SendChar('G');
  SendChar(0x03);
  SendChar(0x0FA);

  counter = 10000;
  if (stav & 0x01){
    while ((ADhigher())&&(counter>0)) counter--;
    if (counter) while ((ADlower())&&(counter>0)) counter--;
  } else {
    while ((ADlower())&&(counter>0)) counter--;
    if (counter) while ((ADhigher())&&(counter>0)) counter--;
  }
  if (counter){    
// Nastaveni casovace
    MTIMSC = 0x30;
    MTIMMOD = 0x85;
    MTIMCLK = 0x07;  
    MTIMSC = 0x00;

    for (i = 0; i<250; i++){
      pole[i] = ADCRL;
      if (stav & 0x2){        
        if (pole[i]>(byte)200) SendChar(10);
        else SendChar((byte)((byte)200-(byte)pole[i])+(byte)10);
      } else{
        n = ((pole[i] * 158)>>8) * 2;        
        if (n>(byte)200) SendChar(10);
          else SendChar((byte)((byte)200-(byte)n)+(byte)10);
      }
      for (n = 0; n<t ; n++){
        while (!(MTIMSC & 0x80)) SRS = 42;
        MTIMSC = 0;
      }
      
    }
    MTIMSC = 0x30;
  } else {
    if (stav & 0x01) for (i = 0; i<250; i++) SendChar(210);
      else for (i = 0; i<250; i++) SendChar(10);
    SendMsg("No Synchro");
  }

 return((byte)counter);  
}

byte ADSynchro(void) {
  byte i;
 
 
  switch (timebase){
    case  0 : i = ADquickSynchro();      break;
    case  1 : i = ADnormalSynchro(1);    break;
    case  2 : i = ADnormalSynchro(4);    break;
    case  3 : i = ADnormalSynchro(9);    break;
    case  4 : i = ADnormalSynchro(19);   break;
    case  5 : i = ADnormalSynchro(49);   break;
    case  6 : i = ADnormalSynchro(99);   break;
    case  7 : i = ADnormalSynchro(199);  break;
    case  8 : i = ADslowSynchro();       break;
    case  9 : i = ADveryslowSynchro(1);  break;
    case 10 : i = ADveryslowSynchro(2);  break;
    case 11 : i = ADveryslowSynchro(4);  break;
    case 12 : i = ADveryslowSynchro(10); break;
    case 13 : i = ADveryslowSynchro(20); break;
    case 14 : i = ADveryslowSynchro(40); break;
  }
  return i;

} 



// Nejrychlejsi sejmuti signalu
void ADquickFree() {
  word ukazat;
  
  // Pretypuji adresu pole (prekladac v ASM casti to odmital)
  ukazat=(unsigned int)pole;

  asm {  
    ; Zakaz preruseni
            CLI
    ; Priprav ukazatel na zacatek pole
            LDHX   ukazat  

    ; "A" registr pouziju jako pocitadlo      
    MERENI: LDA    #$FA       ; Pocitadlo nastaveno na 250
    ; A zaznamenam data (po 17 Bus cyklech) do pameti

    LOOP:   MOV    ADCRL,X+   ; 5 Bus cycles
            NOP               ; 1 (1)
            NOP               ; 1 (2)
            NOP               ; 1 (3)
            NOP               ; 1 (4)
            NOP               ; 1 (5)
            NOP               ; 1 (6)
            NOP               ; 1 (7)
            NOP               ; 1 (8)
            DBNZA   LOOP      ; 4 Bus cycles
    ; Povolim preruseni
            SEI
    }
}

// Sejmuti signalu se spomalenim podle konstanty
void ADnormalFree(byte speed) {
  word ukazat;
  byte i;
  
  // Pretypuji adresu pole (prekladac v ASM casti to odmital)
  ukazat=(unsigned int)pole;

  asm {  
    ; Zakaz preruseni
            CLI
    ; Priprav ukazatel na zacatek pole
            LDHX   ukazat  
    ; "A" registr pouziju jako pocitadlo      
            LDA    #$FA       ; Pocitadlo nastaveno na 250
    ; A zaznamenam data (po 17 Bus cyklech) do pameti


      LOOPL:  MOV    ADCRL,X+   ; 5 Bus cycles

              STA    i          ; 5
              LDA    speed      ; 5 (15)

              NOP               ; 1 (1)
              NOP               ; 1 (2)
              NOP               ; 1 (3)
              NOP               ; 1 (4)
              NOP               ; 1 (5)
              NOP               ; 1 (6)
              NOP               ; 1 (7)
              NOP               ; 1 (8)
              NOP               ; 1 (9)
              DECA              ; 1
              BEQ    LOOPLC     ; 3 (25)
              



      LOOPS:  NOP               ; 1 (1)
              NOP               ; 1 (2)
              NOP               ; 1 (3)
              NOP               ; 1 (4)
              NOP               ; 1 (5)
              NOP               ; 1 (6)
              NOP               ; 1 (7)
              NOP               ; 1 (8)
              NOP               ; 1 (9)
              NOP               ; 1 (10)
              NOP               ; 1 (11)
              NOP               ; 1 (12)
              NOP               ; 1 (13)
              DBNZA   LOOPS     ; 4
      LOOPLC: LDA     i         ; 5
              DBNZA   LOOPL     ; 4 Bus cycles (34)
  ; Povolim preruseni
              SEI
    }
}



void ADslowFree(void){
byte i;

// Nastaveni casovace
  MTIMSC = 0x30;
  MTIMMOD = 0x6A;
  MTIMCLK = 0x06;  
  MTIMSC = 0x00;
  for (i = 0; i<250; i++){
    pole[i] = ADCRL;
    while (!(MTIMSC & 0x80)) SRS = 42;
    MTIMSC = 0;    
  }
  MTIMSC = 0x30;
  
}


void ADveryslowFree(byte t){
byte i,n;

// Nastaveni zobrazovani nezobrazovane obrazovky
  if (!(stav & 0x08)){
    SendChar(0x1B);  
    SendChar('V');
    if (screen == 1) SendChar('1');
      else SendChar('2');  
  }
  Cekej20ms();


  SendChar(0x1B);
  SendChar('G');
  SendChar(0x03);
  SendChar(0x0FA);
// Nastaveni casovace
  MTIMSC = 0x30;
  MTIMMOD = 0x85;
  MTIMCLK = 0x07;  
  MTIMSC = 0x00;
  for (i = 0; i<250; i++){
    pole[i] = ADCRL;
    if (stav & 0x2){        
      if (pole[i]>(byte)200) SendChar(10);
      else SendChar((byte)((byte)200-(byte)pole[i])+(byte)10);
    } else{
      n = ((pole[i] * 158)>>8) * 2;        
      if (n>(byte)200) SendChar(10);
        else SendChar((byte)((byte)200-(byte)n)+(byte)10);
    }
    for (n = 0; n<t ; n++){
      while (!(MTIMSC & 0x80)) SRS = 42;
      MTIMSC = 0;
    }
    
  }
  MTIMSC = 0x30;
  
}


void ADFree(void) {
  switch (timebase) {
    case  0 : ADquickFree();      break;
    case  1 : ADnormalFree(1);    break;
    case  2 : ADnormalFree(4);    break;
    case  3 : ADnormalFree(9);    break;
    case  4 : ADnormalFree(19);   break;
    case  5 : ADnormalFree(49);   break;
    case  6 : ADnormalFree(99);   break;
    case  7 : ADnormalFree(199);  break;
    case  8 : ADslowFree();       break;
    case  9 : ADveryslowFree(1);  break;
    case 10 : ADveryslowFree(2);  break;
    case 11 : ADveryslowFree(4);  break;
    case 12 : ADveryslowFree(10); break;
    case 13 : ADveryslowFree(20); break;
    case 14 : ADveryslowFree(40); break;
  }
}


// ********************************************************
// *                      FUNKCE GRAF                     *
// ********************************************************

void HoldValue(byte X);

void Ramec(void);

void Graf(void) {

 byte i=0;
 word n;
// Nastaveni zapisu na nezobrazovanou obrazovku a vymalovani Ramce
  SendChar(0x1B);  
  SendChar('W');
  if (screen == 1) {
    SendChar('2');
    screen = 0;  
  }
    else {
    screen = 1;  
    SendChar('1');
  }

  Ramec();


// Provedeni mereni
  if (!(stav & 0x08)){
    if (stav & 0x4) ADFree();
      else i=ADSynchro();
    mertimebase=timebase;
    mergain=gain;
  }


// Nastaveni zobrazovani nezobrazovane obrazovky pokud se merilo  
  if ((!(stav & 0x08))&&(timebase<9)){
    SendChar(0x1B);  
    SendChar('V');
    if (screen == 1) SendChar('1');
      else SendChar('2');  
  }
  Cekej20ms();


// Vykresleni hodnot 
 if (((!i)&&(timebase<9))||(stav & 0x08)) {  
   SendChar(0x1B);
   SendChar('G');
   SendChar(0x03);
   SendChar(0x0FA);
   for (i=0 ; i<0x0FA ; i++) {
      if (stav & 0x2){        
        if (pole[i]>(byte)200) SendChar(10);
        else SendChar((byte)((byte)200-(byte)pole[i])+(byte)10);
      } else{
        n = ((pole[i] * 158)>>8) * 2;        

        if (n>(byte)200) SendChar(10);
        else SendChar((byte)((byte)200-(byte)n)+(byte)10);
      
      }
   }
   Cekej100ms();
 }else if (timebase<9) SendMsg("No Synchro");

// Vykresleni "hold" cary
 if (stav & 0x08) {
   SendChar(0x1B);
   SendChar('U');
   SendChar((byte)3+(byte)Xhold);
   SendChar((byte)10);
   SendChar((byte)200);
   if (stav & 0x02) HoldValue(pole[Xhold]);
  }

// Nastaveni zobrazovani nezobrazovane obrazovky pokud se nemerilo
  if (stav & 0x08){
    SendChar(0x1B);  
    SendChar('V');
    if (screen == 1) SendChar('1');
      else SendChar('2');  
  }
  Cekej20ms();
}


void HoldValue(byte X) {
byte i;
word Y,zbytek;

  SendChar(0x1B);  
  SendChar('Y');
  SendChar(30);
  SendChar(0x1B);  
  SendChar('X');
  SendChar(0);

  if (X>210) X=210;

  SendMsg("Voltage: ");
  if (X<100){
    SendChar('-');
    X = (unsigned)100 - (unsigned)X;
  } else X = (unsigned)X - (unsigned)100;
  
  zbytek = 0;
  switch (mergain){
    case  0 : Y = (X<<2)/10; break;
    case  1 : Y = (X<<3)/10; break;
    case  2 : Y = (X<<1);    break;
    case  3 : Y = (X<<2);    break;
    case  4 : Y = (X<<3);    break;
    case  5 : {
      Y = (X<<1)/100;
      zbytek = (X<<1)%100;
    }break;
    case  6 : {
      Y = (X<<2)/100;
      zbytek = (X<<2)%100;
    }break;
    case  7 : {
      Y = (X<<3)/100;
      zbytek = (X<<3)%100;
    }break;
    case  8 : {
      Y = (X<<1)/10;
      zbytek = (X<<1)%10;
    }break;
    case  9 : {
      Y = (X<<2)/10;
      zbytek = (X<<2)%10;
    }break;
    case 10 : Y = (X<<3)/10;  break;
    case 11 : Y = (X<<1);     break;
  }


  
  i=((byte)(Y/100));
  if (i) SendChar(i|0x30);    
  i=((byte)((Y%10)%10))|0x30;
  if (Y>9) SendChar(i);    
  i=((byte)(Y%10))|0x30;
  SendChar(i);    

  if (zbytek) {
    SendChar('.');
    SendChar((zbytek/10)|0x30);    
    SendChar((zbytek%10)|0x30);    

  }
  if (mergain>4) SendMsg(" V");
    else SendMsg(" mV");
    

}


void ClrPozadi(void) {

 byte i=0;

// Nastaveni zapisu na nezobrazovanou obrazovku a vymalovani Ramce
  SendChar(0x1B);  
  SendChar('W');
  if (screen == 1) {
    SendChar('2');
    screen = 0;  
  }
    else {
    screen = 1;  
    SendChar('1');
  }

  Ramec();

  SendChar(0x1B);  
  SendChar('V');
  if (screen == 1) SendChar('1');
    else SendChar('2');  
  Cekej20ms();
}


// ********************************************************
// *                 UVODNI SYNCHRONIZACE                 *
// ********************************************************

void Synchro(void) 
{
  
  byte horni,dolni,i,n,trimh,triml;
  word trim;
  char znak[4];

// Defaultni hodnota trimovaciho registru po resetu  
  trimh=0x80;
  triml=0x00;
  
// Provede se 50 pokusu o nastaveni
  for (n=0; n<50; n++){
// Testovaci smycka v ASM pro jeden impuls 2.5 ms
    asm{
    ; Zakaz preruseni
            CLI
    ; Vynulovani registru
            CLRA
            CLRX
    ; Cekani na nulovy signal
    NULA:   BRSET   2,PTBD,NULA
    ; Cekani na vzestupnou hranu
    JEDNA:  BRCLR   2,PTBD,JEDNA
    ; Pocitaci smycka
    LOOP:   DBNZA   LOOPC
            INCX
    LOOPC:  BRSET   2,PTBD,LOOP      
    ; Ulozeni navratovych hodnot
            STA     dolni
            STX     horni
    ; Povolim preruseni
            SEI
    }

// Prepocet namerene hodnoty
    trim=((word)0x100 - (word)dolni)+((word)horni)*0x100;
    if (trim == 2360) break;

// Test zda se ma trimovaci registr zvednout ci snizit a provedeni
// Spravna hodnota poctu pruchodu smyckou "LOOP" ma byt 2360
    if (trim>2360) {
      if (triml==1) {
        triml=0;
        ICSSC=0;
        trimh++;
        ICSTRM=trimh;        
      } else{
        triml=1;
        ICSSC=1;        
      }
    } else {
      if (triml==0) {
        triml=1;
        ICSSC=1;
        trimh--;
        ICSTRM=trimh;        
      } else{
        triml=0;
        ICSSC=0;        
      }
    }
  }
// Vymazani obrazovky
  SendChar(0x1B);
  SendChar('S');
  Cekej10ms();  
// Vypis hlaseni
  if ((trim>2350) && (trim<2370)) SendMsg("Time calibration OK - ");
    else SendMsg("Error time calibration - ");
// Vypis namerene hodnty
  for (i=0; i<4; i++) {
    znak[3-i]=(trim%10)|0x30;
    trim=trim/10;
  }
  for (i=0; i<4; i++) SendChar(znak[i]);
}


// ********************************************************
// *                   POZADI OBRAZOVKY                   *
// ********************************************************

void Ramec(void) {
  int i;

// Vymazani obrazovky
  SendChar(0x0C);
  Cekej10ms();

// Nastaveni barvy
  SendChar(0x1B);
  SendChar('C');
  SendChar(0x0F);

// Vykresleni krize
  for (i=0; i<51; i++){
    SendChar(0x1B);
    SendChar('U');
    SendChar(i*5+3);
    SendChar(108);
    SendChar(5);
  }
  for (i=0; i<41; i++){
    SendChar(0x1B);
    SendChar('T');
    SendChar(126);
    SendChar(i*5+10);
    SendChar(5);
  }

  
// Vykresleni vodorovnych car
  for (i=0; i<9; i++){
    SendChar(0x1B);
    SendChar('T');
    SendChar(3);
    SendChar(i*25+10);
    SendChar(250);
  }

// Vykresleni svislych car
  for (i=0; i<11; i++){
    SendChar(0x1B);
    SendChar('U');
    SendChar(i*25+3);
    SendChar(10);
    SendChar(200);    
  }
  Cekej100ms();

// Nastaveni barvy na nejsvetlejsi
  SendChar(0x1B);
  SendChar('C');
  SendChar(0xF0);

// Vykresleni triggeru
  SendChar(0x1B);
  SendChar('U');
  SendChar((byte)3);
  if (stav & 0x02) {
    if (trigger>=100) {
      SendChar((byte)210 - (byte)trigger);
      SendChar((byte)trigger - (byte)100);  
    } else{
      SendChar((byte)110);
      SendChar((byte)100 - (byte)trigger);   
    }
  } else {
    SendChar((byte)210 - (byte)(((trigger * 158)>>8) * 2));
    SendChar(((trigger * 158)>>8) * 2);
  }

// Nastaveni barvy na nejsvetlejsi
  SendChar(0x1B);
  SendChar('C');
  SendChar(0xFF);

// Popisy
  SendMsg("Oscilloscope            http://radim.xf.cz");
  SendChar(0x0D);
// Nastaveni spodniho radku
  SendChar(0x1B);
  SendChar('Y');
  SendChar(27);
// Vypis casove zakladny
  SendMsg("Timebase: ");
  if (stav & 0x08) i=mertimebase;
    else i=timebase;
  switch (i){
    case  0 : SendMsg(" 50us/div"); break;
    case  1 : SendMsg("100us/div"); break;
    case  2 : SendMsg("200us/div"); break;
    case  3 : SendMsg("500us/div"); break;
    case  4 : SendMsg("  1ms/div"); break;
    case  5 : SendMsg("  2ms/div"); break;
    case  6 : SendMsg("  5ms/div"); break;
    case  7 : SendMsg(" 10ms/div"); break;
    case  8 : SendMsg(" 20ms/div"); break;
    case  9 : SendMsg(" 50ms/div"); break;
    case 10 : SendMsg("100ms/div"); break;
    case 11 : SendMsg("200ms/div"); break;
    case 12 : SendMsg("500ms/div"); break;
    case 13 : SendMsg("  1 s/div"); break;
    case 14 : SendMsg("  2 s/div"); break;
  }
// Vypis horizontalniho zesileni
  SendMsg("        Gain: ");
  if (stav & 0x08) i=mergain;
    else i=gain;

  if (stav & 0x02){
    switch (i){
      case  0 : SendMsg(" 10mV/div"); break;
      case  1 : SendMsg(" 20mV/div"); break;
      case  2 : SendMsg(" 50mV/div"); break;
      case  3 : SendMsg("100mV/div"); break;
      case  4 : SendMsg("200mV/div"); break;
      case  5 : SendMsg("500mV/div"); break;
      case  6 : SendMsg("  1 V/div"); break;
      case  7 : SendMsg("  2 V/div"); break;
      case  8 : SendMsg("  5 V/div"); break;
      case  9 : SendMsg(" 10 V/div"); break;
      case 10 : SendMsg(" 20 V/div"); break;
      case 11 : SendMsg(" 50 V/div"); break;
    }
  } else{
    switch (i){
      case  0 : SendMsg("  5mV/div"); break;
      case  1 : SendMsg(" 10mV/div"); break;
      case  2 : SendMsg(" 25mV/div"); break;
      case  3 : SendMsg(" 50mV/div"); break;
      case  4 : SendMsg("100mV/div"); break;
      case  5 : SendMsg("250mV/div"); break;
      case  6 : SendMsg("500mV/div"); break;
      case  7 : SendMsg("  1 V/div"); break;
      case  8 : SendMsg("2.5 V/div"); break;
      case  9 : SendMsg("  5 V/div"); break;
      case 10 : SendMsg(" 10 V/div"); break;
      case 11 : SendMsg(" 25 V/div"); break;
    }  
  }
// Vypis druheho radku
  if (stav & 0x08) {
    SendMsg("HOLD");
    SendChar('\r');
  } else{
    if (stav & 0x02) SendMsg("Bipolar   ");
      else SendMsg("Unipolar  ");
    if (stav & 0x04) SendMsg("No Synchro   ");
      else if (stav & 0x01) SendMsg("Synchro  Up  ");
        else  SendMsg("Synchro Down ");
    if (stav & 0x10) SendMsg("    One shot ");
      else SendMsg("    Free run ");
    if (stav & 0x20) SendMsg("    AC");
      else SendMsg("    DC");
  }
// Vypis tretiho radku s Help
  SendChar(0x1B);
  SendChar('C');
  SendChar(0xF0);
  SendMsg("                                  [H] Help");
  SendChar(0x1B);
  SendChar('C');
  SendChar(0xFF);
}

// ********************************************************
// *                  INFORMATIVNI FUNKCE                 *
// ********************************************************

void Help(void){
  int i;

  SendChar(0x1B);  
  SendChar(0x09);

  SendChar(0x1B);  
  SendChar('S');
  
  Cekej100ms();

  SendMsg("\r Oscilloscope - Help\r\r");
  SendMsg("  /\\   - Gain + \r");
  SendMsg("  \\/   - Gain - \r\r");
  SendMsg("  ->   - Timebase + \r");
  SendMsg("  <-   - Timebase - \r\r");

  SendMsg("   S   - NoSynchro/SynchroDown/SynchroUp\r");
  SendMsg("   u   - Trigger + 0.2 div\r");
  SendMsg("   U   - Trigger +   1 div\r");
  SendMsg("   d   - Trigger - 0.2 div\r");
  SendMsg("   D   - Trigger -   1 div\r\r");

  SendMsg(" space - Hold data \r");
  SendMsg("   >   - Shift cursor right \r");
  SendMsg("   <   - Shift cursor left \r\r");

  SendMsg("   P   - Bipolar/Unipolar \r\r");

  SendMsg("   A   - AC/DC \r\r");

  SendMsg("   F   - Freerun/One shot \r");
  SendMsg(" enter - Start one shot \r\r");

  SendMsg("   C   - Center screen \r\r");

  SendMsg("   H   - Help \r\r");

  SendChar(0x1B);
  SendChar('U');
  SendChar(0);
  SendChar(0);
  SendChar(255);
  SendChar(0x1B);
  SendChar('U');
  SendChar(254);
  SendChar(0);
  SendChar(255);

  SendChar(0x1B);
  SendChar('T');
  SendChar(0);
  SendChar(0);
  SendChar(255);
  SendChar(0x1B);
  SendChar('T');
  SendChar(0);
  SendChar(255);
  SendChar(255);

  for (i=0; i<50; i++) Cekej100ms();
  SendChar(0x1B);  
  SendChar(0x09);

}

void Znak(byte x, byte y, byte z){
  SendChar(0x1B);  
  SendChar('Z');
  SendChar(x);
  SendChar(y);
  SendChar(z);
}

void Uvodtxt(void){
  byte i;

  SendChar(0x1B);  
  SendChar('W');
  SendChar('2');
  Cekej20ms();

  SendChar(0x1B);  
  SendChar('S');
  Cekej100ms();
  SendChar(0x1B);  
  SendChar('V');
  SendChar('2');

  SendChar(0x1B);  
  SendChar('C');
  SendChar(0xFF);

  Znak(100,36,'n');
  Znak(126,36,'f');
  SendChar(0x1B);  
  SendChar('Y');
  SendChar(10);
  SendMsg("              OSCILLOSCOPE");
  SendChar(0x1B);  
  SendChar('Y');
  SendChar(16);
  SendChar(0x1B);  
  SendChar('C');
  SendChar(0x0F);

  SendMsg("\r  Timebase: 50us/div ..  2s/div ");
  SendMsg("\r  Gain:     10mV/div .. 50V/div");
  SendMsg("\r  Input:    1MOhm/30pF");
  SendChar(0x1B);  
  SendChar('C');
  SendChar(0xFF);
                   
  SendMsg("\r\r\r\r                         (c) Radim Pechal");
  SendMsg("\r                   radim.pechal@seznam.cz");
  SendMsg("\r                       http://radim.xf.cz");


  SendChar(0x1B);
  SendChar('U');
  SendChar(0);
  SendChar(0);
  SendChar(255);
  SendChar(0x1B);
  SendChar('U');
  SendChar(254);
  SendChar(0);
  SendChar(255);

  SendChar(0x1B);
  SendChar('T');
  SendChar(0);
  SendChar(0);
  SendChar(255);
  SendChar(0x1B);
  SendChar('T');
  SendChar(0);
  SendChar(255);
  SendChar(255);    

  for (i=1; i<50; i++) Cekej100ms();

  SendChar(0x1B);  
  SendChar('S');
  Cekej100ms();

}

// ********************************************************
// *                 FUNKCE PRO CENTROVANI                *
// ********************************************************

void Centrovani(void){
  byte znak;
  int i;


  SendChar(0x1B);  
  SendChar('W');
  if (screen == 1) {
    SendChar('2');
    screen = 0;  
  }
    else {
    screen = 1;  
    SendChar('1');
  }

// Vymazani obrazovky
  SendChar(0x0C);
  Cekej10ms();

  SendChar(0x1B);
  SendChar('C');
  SendChar(0x0F);

  for (i=1; i<4; i++){
    SendChar(0x1B);
    SendChar('U');
    SendChar(i*64);
    SendChar(0);
    SendChar(255);
    SendChar(0x1B);
    SendChar('T');
    SendChar(0);
    SendChar(i*64);
    SendChar(255);
    }

  SendChar(0x1B);
  SendChar('C');
  SendChar(0xFF);

  SendChar(0x1B);
  SendChar('U');
  SendChar(0);
  SendChar(0);
  SendChar(255);
  SendChar(0x1B);
  SendChar('U');
  SendChar(254);
  SendChar(0);
  SendChar(255);

  SendChar(0x1B);
  SendChar('T');
  SendChar(0);
  SendChar(0);
  SendChar(255);
  SendChar(0x1B);
  SendChar('T');
  SendChar(0);
  SendChar(255);
  SendChar(255);    

  SendChar(0x1B);
  SendChar('X');
  SendChar(10);

  SendChar(0x1B);
  SendChar('Y');
  SendChar(15);
  SendMsg("Setting screen posicion\r");

  SendChar(0x1B);
  SendChar('X');
  SendChar(1);

  SendChar(0x1B);
  SendChar('Y');
  SendChar(30);
  SendMsg(" Accept [Enter] Storno [Esc]");
  
  SendChar(0x1B);  
  SendChar('V');
  if (screen == 1) SendChar('1');
    else SendChar('2');  
  Cekej20ms();

  while (znak !='\r'){
    SendChar(0x1B);
    SendChar('H');
    SendChar(obrX);
    SendChar(0x1B);
    SendChar('O');
    SendChar(obrY);

    while (!SCIS1_RDRF) {
      feedCOP();
    }
    znak = SCID;    
    switch (znak){
      case '2': if (obrY <= 14) obrY++;break;
      case '8': if (obrY >=  1) obrY--;break;
      case '6': if (obrX <= 14) obrX++;break;
      case '4': if (obrX >=  1) obrX--;break;
    }

  }

}


// ********************************************************
// *                NASTAVENI PINU PRO GAIN               *
// ********************************************************

void UpravGain(void) {
  switch (gain){
    case 0  : {PTBD_PTBD5 = 0; PTBD_PTBD4 = 1; PTBD_PTBD3 = 1; PTBD_PTBD7 = 0;} break;
    case 1  : {PTBD_PTBD5 = 0; PTBD_PTBD4 = 0; PTBD_PTBD3 = 1; PTBD_PTBD7 = 0;} break;
    case 2  : {PTBD_PTBD5 = 1; PTBD_PTBD4 = 1; PTBD_PTBD3 = 0; PTBD_PTBD7 = 0;} break;
    case 3  : {PTBD_PTBD5 = 1; PTBD_PTBD4 = 0; PTBD_PTBD3 = 0; PTBD_PTBD7 = 0;} break;
    case 4  : {PTBD_PTBD5 = 0; PTBD_PTBD4 = 1; PTBD_PTBD3 = 0; PTBD_PTBD7 = 0;} break;
    case 5  : {PTBD_PTBD5 = 0; PTBD_PTBD4 = 0; PTBD_PTBD3 = 0; PTBD_PTBD7 = 0;} break;
    case 6  : {PTBD_PTBD5 = 0; PTBD_PTBD4 = 1; PTBD_PTBD3 = 1; PTBD_PTBD7 = 1;} break;
    case 7  : {PTBD_PTBD5 = 0; PTBD_PTBD4 = 0; PTBD_PTBD3 = 1; PTBD_PTBD7 = 1;} break;
    case 8  : {PTBD_PTBD5 = 1; PTBD_PTBD4 = 1; PTBD_PTBD3 = 0; PTBD_PTBD7 = 1;} break;
    case 9  : {PTBD_PTBD5 = 1; PTBD_PTBD4 = 0; PTBD_PTBD3 = 0; PTBD_PTBD7 = 1;} break;
    case 10 : {PTBD_PTBD5 = 0; PTBD_PTBD4 = 1; PTBD_PTBD3 = 0; PTBD_PTBD7 = 1;} break;
    case 11 : {PTBD_PTBD5 = 0; PTBD_PTBD4 = 0; PTBD_PTBD3 = 0; PTBD_PTBD7 = 1;} break;
  }
  Cekej20ms();
}
  
// ********************************************************
// *                        MAIN                          *
// ********************************************************
void main(void) {
  byte znak;
  
  EnableInterrupts; //Povoleni preruseni  

// Inicializace
// Nastaveni vstupu/vystupu
  PTAPE = 0xFF;
  PTBPE = 0xFF;
  PTBDD_PTBDD7 = 1;         //Pin pro zapnuti/vypnuti delice
  PTBD_PTBD7 = 1;
  PTBDD_PTBDD6 = 1;         //Pin pro prepinani unipolar/bipolar
  PTBD_PTBD6 = 0;           
  PTBDD_PTBDD5 = 1;         //Pin pro nastaveni zesileni
  PTBD_PTBD5 = 0;           
  PTBDD_PTBDD4 = 1;         //Pin pro nastaveni zesileni
  PTBD_PTBD4 = 1;
  PTBDD_PTBDD3 = 1;         //Pin pro nastaveni zesileni
  PTBD_PTBD3 = 1;
  PTADD_PTADD2 = 1;         //Pin pro prepinani AC/DC
  PTAD_PTAD2 = 1;  
  PTBDD_PTBDD2 = 0;         //Pin se signalem na pocatecni synchro

// Nastaveni ICS na FEI mode
  ics_intclk(0x04,0x00);
// Nastaveni SCI pro 9600 baud
  SCIBD = fei1_spd;
  SCIC1 = 0x10;
  SCIC2 = 0x0C;
  SCIC3 = 0x40;
// Nastaveni A/D converter
  APCTL1 = 0x03;            // Port PTA0 a PTA1 jako A/D
  ADCCFG = 0x00;            // Rychlost a 8bit. prevod
  ADCSC1 = 0x20;            // Continuous a kanal 0
  ADCSC2 = 0x00;            // Vypnout komparaci
  ADCSC1 = 0x20;            // Continuous a kanal 0

// Sesynchronizovani na 17MHz  
  Synchro();
  Cekej100ms();



// Vypis uvodni obrazovky
  Uvodtxt();

// ********************************************************
// *                  HLAVNI SMYCKA                       *
// ********************************************************
  for(;;) {
    if ((!(stav & 0x10))||(stav & 0x08)) Graf();
      else if (!(znak == '\r')) ClrPozadi();

    feedCOP();  /*feeds the dog */

// Obsluha Hold
    if (stav & 0x18) {
// Ocekavani klavesy pri Hold nebo pri One shot
      while (!SCIS1_RDRF) {
        feedCOP();
      }
    } else {  
// Test zda neprisla klavesa
      if (!SCIS1_RDRF) {
        continue;
      } 
    }


// Vyhodnoceni prijate klavesy    
    znak = SCID;    
    switch (znak) {
// Help
      case 'h' : case 'H' : Help(); break;      
// Centrovani obrazovky
      case 'c' : case 'C' : Centrovani(); break;      
// Prepinani AC/DC
      case 'a' : case 'A' : if (stav & 0x20) {
        PTAD_PTAD2 = 1; 
        stav &= (byte)0xDF;
      } else{
        PTAD_PTAD2 = 0; 
        stav |= (byte)0x20;
      }break;
// One shot / Free run
      case 'f' : case 'F' : if (stav & 0x10){
        stav &= 0xEF;
      } else {
        ClrPozadi();
        stav |= 0x10;   
      }break;      
      case '\r' : if ((stav & 0x10)&&(!((stav & 0x08)))) Graf(); break;
// Polarita signalu (Unipolar / Bipolar)
      case 'p' : case 'P' : if (stav & 0x2) {
        PTBD_PTBD6 = 1; 
        stav &= (byte)0xFD;
        if (trigger>160) trigger=160;
      } else{
        PTBD_PTBD6 = 0; 
        stav |= (byte)0x2;
      }break;
// Obsluha hold
      case ' ' : if (stav & 0x08) stav &= 0xF7;
        else stav |= 0x08; break;
      case ',' : if (Xhold > 0) Xhold -= 5; break;
      case '.' : if (Xhold < 250) Xhold += 5; break;
// Nastaveni Synchra
      case 'S' : case 's' : {
        if (stav & 0x04) stav &= 0xFA;
          else if (stav & 0x01) stav |= 0x04;
            else stav |= 0x01;
      }break;
// Nastavovani zesileni
      case '8' : if (gain < 11) {
        gain++;
        UpravGain();
      } break;      
      case '2' : if (gain > 0) {
        gain--; 
        UpravGain();      
      } break;      
// Nastavovani casove zakladny
      case '6' : if (timebase < 14) timebase++; break;      
      case '4' : if (timebase > 0) timebase--; break;      
// Nastavovani Triggeru
      case 'U' : if (stav & 0x02) {
          if (trigger <= 175) trigger+=25;
          }
          else if (trigger <= 135) trigger+=25; break;      
      case 'u' : if (stav & 0x02) {
          if (trigger <= 195) trigger+=5;
          }
          else if (trigger <= 155) trigger+=5; break;            
      case 'D' : if (trigger >= 25) trigger-=25; break;      
      case 'd' : if (trigger >= 5) trigger-=5; break;      
// Spatne zadani
      default: SendMsg("\rInvalid selection");
    }
  }

}

// A to je vse :)